rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_comparing_statistics.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script shows how to compare different set of 
rem                     statistics with the package DBMS_STATS.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: The user executing the script must have the privilege
rem                     to execute the package DBMS_LOCK. If the privilege is
rem                     not available, manually wait 15 seconds...
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF LONG 1000000 PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t
AS
SELECT rownum AS id,
       round(dbms_random.normal*1000) AS val1,
       10 + round(ln(rownum/2+2)) AS val2,
       dbms_random.string('p',25) AS pad
FROM all_objects
WHERE rownum <= 10000
ORDER BY dbms_random.value;

ALTER TABLE t ADD CONSTRAINT t_pk PRIMARY KEY (id);

PAUSE

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 1,
    method_opt       => 'for all columns size skewonly',
    cascade          => TRUE
  );
END;
/

PAUSE

REM
REM insert more data
REM

INSERT INTO t
SELECT 10000+id, val1, val2, pad
FROM t
WHERE rownum <= 2500
ORDER BY dbms_random.value;

COMMIT;

PAUSE

REM
REM gather pending object statistics
REM

BEGIN
  dbms_stats.set_table_prefs(user,'t','publish','false');
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 1,
    method_opt       => 'for all columns size skewonly',
    cascade          => TRUE
  );
  dbms_stats.set_table_prefs(user,'t','publish','true');
END;
/

PAUSE

REM
REM compare current statistics with pending statistics
REM

SELECT *
FROM table(dbms_stats.diff_table_stats_in_pending(ownname => user,
                                                  tabname => 'T', 
                                                  time_stamp => NULL,
                                                  pctthreshold => 10));

PAUSE

REM
REM wait 15 seconds and then publish pending statistics
REM

execute dbms_lock.sleep(15)

execute dbms_stats.publish_pending_stats(ownname => user, tabname => 'T');

PAUSE

REM
REM compare current statistics with statistics of 15 seconds ago
REM

SELECT *
FROM table(dbms_stats.diff_table_stats_in_history(
             ownname      => user,
             tabname      => 'T',
             time1        => systimestamp - to_dsinterval('0 00:00:15'),
             time2        => NULL,
             pctthreshold => 10));


SET ECHO OFF
