rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: DBA
rem  Script-Name......: ex_execution_plan_stability_10g.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script, along with the other one with the suffix
rem                     "11g" is used to demonstrate how to create a SQL
rem                     tuning set in 10gR2, move it to 11gR1 and create an
rem                     execution plan baseline based on it.
rem  Usage............: 1) 10gR2: run ex_execution_plan_stability_10g.sql
rem                     2) 10gR2: export the staging table 
rem                     3) 11gR1: import the staging table
rem                     4) 11gR1: run ex_execution_plan_stability_11g.sql
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF LONG 100000 PAGESIZE 100 LINESIZE 100

COLUMN PLAN_TABLE_OUTPUT FORMAT A80
COLUMN SQL_TEXT FORMAT A45
COLUMN SQL_HANDLE FORMAT A25

DROP TABLE t PURGE;

DROP TABLE my_stgtab PURGE;

execute dbms_sqltune.drop_sqlset('10GR2_SET')

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t (id, n, pad, CONSTRAINT t_pk PRIMARY KEY (id))
AS 
SELECT rownum, rownum, rpad('*',500,'*')
FROM dba_objects WHERE rownum <= 1000;

BEGIN
  dbms_stats.gather_table_stats(
    ownname => user, 
    tabname => 't', 
    estimate_percent => 100, 
    method_opt => 'for all columns size 254',
    cascade => TRUE
  );
END;
/

PAUSE

REM
REM run few times a test query
REM

SELECT count(pad) FROM t WHERE n = 42;

SELECT count(pad) FROM t WHERE n = 42;

SELECT count(pad) FROM t WHERE n = 42;

PAUSE

REM
REM create a SQL tuning set containing the test query
REM

DECLARE
  baseline_cursor DBMS_SQLTUNE.SQLSET_CURSOR;
BEGIN
  dbms_sqltune.create_sqlset(
    sqlset_name => '10GR2_SET'
  );
  
  OPEN baseline_cursor FOR
    SELECT value(p)
    FROM table(dbms_sqltune.select_cursor_cache(
                 'sql_id=''2n8tjyr1nthf5''', -- basic_filter
                 NULL,                       -- object_filter
                 NULL, NULL, NULL,           -- ranking_measure1/2/3
                 NULL,                       -- result_percentage
                 NULL,                       -- result_limit
                 'all'                       -- attribute_list
               )) p;
    
  dbms_sqltune.load_sqlset(
    sqlset_name => '10GR2_SET', 
    populate_cursor => baseline_cursor
  );
END;
/

PAUSE

REM
REM display content of the SQL tuning set
REM

SELECT * 
FROM table(dbms_xplan.display_sqlset('10GR2_SET', '2n8tjyr1nthf5'));

PAUSE

REM
REM export SQL tuning set
REM

BEGIN
  dbms_sqltune.create_stgtab_sqlset(
    table_name  => 'MY_STGTAB',
    schema_name => user
  );
  dbms_sqltune.pack_stgtab_sqlset(
    sqlset_name          => '10GR2_SET',
    sqlset_owner         => user,
    staging_table_name   => 'MY_STGTAB',
    staging_schema_owner => user
  );
END;
/

PAUSE

REM
REM export staging table with Data Pump
REM

REM expdp tables=MY_STGTAB

PAUSE

REM
REM cleanup
REM

REM remove SQL tuning set

execute dbms_sqltune.drop_sqlset('10GR2_SET')

REM remove stating table

DROP TABLE my_stgtab PURGE;

SET ECHO OFF
