rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_extended_statistics2.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script shows how to enhance the query optimizer 
rem                     estimations with extended statistics. The example uses
rem                     two columns with correlated data.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80 NUMWIDTH 20

DROP TABLE t PURGE;

COLUMN extension FORMAT A40

CLEAR SCREEN

REM
REM create test table with data and gather object statistics
REM

CREATE TABLE t 
AS
SELECT rownum AS id, mod(rownum,10) AS n1, mod(rownum,10) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 254'
  );
END;
/

PAUSE

REM
REM shows the correlation between column n1 and column n2
REM

SELECT count(*) FROM t WHERE n1 = 5;

SELECT count(*) FROM t WHERE            n2 = 5;

SELECT count(*) FROM t WHERE n1 = 5 AND n2 = 5;

PAUSE

REM
REM the query optimizer does a correct estimation with
REM the predicates "n1 = 5" and "n2 = 5"
REM

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE n1 = 5;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE n2 = 5;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM since data of column n1 and column n2 is correlated the
REM query optimizer is not able to correctly estimate the
REM cardinality of a predicate based on both columns
REM

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE n1 = 5 AND n2 = 5;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM create a extension to provide addition information
REM to the query optimizer
REM

SELECT dbms_stats.create_extended_stats(
         ownname   => user,
         tabname   => 'T',
         extension => '(n1,n2)'
       ) AS extension_name
FROM dual;

PAUSE

SELECT extension_name, extension
FROM user_stat_extensions
WHERE table_name = 'T';

PAUSE

REM
REM for the extension a regular virtual column is created
REM

SELECT column_name, virtual_column
FROM user_tab_cols
WHERE table_name = 'T';

PAUSE

SELECT DISTINCT &extension_name FROM t;

PAUSE

SELECT DISTINCT sys_op_combined_hash(n1, n2) FROM t;

PAUSE

REM
REM when object statistics are gathered the virtual
REM column is processed as any other column... therefore,
REM column statistics are gathered for it as well
REM

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 254'
  );
END;
/

PAUSE

REM
REM thanks to the extension the query optimizer is able
REM to correctly estimate the cardinality
REM

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE n1 = 5 AND n2 = 5;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

SET ECHO OFF
