rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_full_outer_join.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: The script shows that with 11g the full outer join 
rem                     syntax is no longer processed with a UNION ALL.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem 05.04.2008 ChA       Added example with hint native_full_outer_join
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;

CLEAR SCREEN

REM
REM create test tables
REM

CREATE TABLE t1 AS 
SELECT rownum AS id 
FROM all_objects
WHERE rownum <= 5;

execute dbms_stats.gather_table_stats(user,'t1')

CREATE TABLE t2 AS 
SELECT rownum+2 AS id
FROM all_objects
WHERE rownum <= 5;

execute dbms_stats.gather_table_stats(user,'t2')

PAUSE

REM
REM without the new optimization a UNION ALL is used
REM to process a full outer join
REM

ALTER SESSION SET "_optimizer_native_full_outer_join" = OFF;

EXPLAIN PLAN FOR
SELECT *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM with the new optimization a UNION ALL is no longer used
REM

ALTER SESSION SET "_optimizer_native_full_outer_join" = FORCE;

PAUSE

EXPLAIN PLAN FOR
SELECT *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM two hints related to full outer joins:
REM - native_full_outer_join
REM - no_native_full_outer_join
REM

EXPLAIN PLAN FOR
SELECT /*+ no_native_full_outer_join */ *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

ALTER SESSION SET "_optimizer_native_full_outer_join" = OFF;

EXPLAIN PLAN FOR
SELECT /*+ native_full_outer_join */ *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM the new optimization works with equi joins only
REM (hash joins support equi joins only!)
REM

EXPLAIN PLAN FOR
SELECT *
FROM t1 FULL OUTER JOIN t2 ON t1.id BETWEEN t2.id AND t2.id+1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

SET ECHO OFF
