rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_invisible_index.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script shows how to change the visibility of an
rem                     index, which the impact of visibility on the query 
rem                     optimizer is and how to take advantage of the
rem                     parameter optimizer_use_invisible_indexes.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

CLEAR SCREEN

ALTER SESSION SET optimizer_use_invisible_indexes = FALSE;

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create a test table (incl. primary key and object statistics)
REM

CREATE TABLE t AS 
SELECT rownum AS id, rpad('*',50,'*') AS pad 
FROM all_objects
WHERE rownum <= 1000;

ALTER TABLE t ADD CONSTRAINT t_pk PRIMARY KEY (id);

execute dbms_stats.gather_table_stats(user,'t')

PAUSE

REM
REM obviously the primary key index is used when a restriction is 
REM based on the column composing it
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(format => 'basic'));

PAUSE

REM
REM change the visibility of the index
REM

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

ALTER INDEX t_pk INVISIBLE;

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

PAUSE

REM
REM check that the query optimizer no longer use the invisible index
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM also when a hint is specified the index cannot be used
REM

EXPLAIN PLAN FOR SELECT /*+ index(t t_pk) */ * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM change the visibility of the index and check its utilization
REM

ALTER INDEX t_pk VISIBLE;

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM through the parameter optimizer_use_invisible_indexes it is 
REM possible to instruct the query optimizer to use an invisible index
REM

ALTER INDEX t_pk INVISIBLE;

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

ALTER SESSION SET optimizer_use_invisible_indexes = TRUE;

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

ALTER SESSION SET optimizer_use_invisible_indexes = FALSE;

set autotrace off

SET ECHO OFF
